import argparse

from blueness import module
from blueness.argparse.generic import sys_exit
from blue_objects.env import ABCLI_OBJECT_ROOT

from blue_geo import NAME
from blue_geo.QGIS.seed import generate_seed
from blue_geo.QGIS.dependency import list_of_dependencies
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

list_of_tasks = "generate_seed|list_dependencies"

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=list_of_tasks,
)
parser.add_argument(
    "--delim",
    type=str,
    default="+",
)
parser.add_argument(
    "--filename",
    type=str,
)
parser.add_argument(
    "--verbose",
    type=int,
    default="0",
    help="0|1",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = args.task in list_of_tasks.split("|")
if args.task == "generate_seed":
    print(generate_seed())
elif args.task == "list_dependencies":
    output = list_of_dependencies(
        filename=args.filename,
        ABCLI_OBJECT_ROOT=ABCLI_OBJECT_ROOT,
        verbose=args.verbose == 1,
    )

    print(delim.join(output))
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import os

if not QGIS_is_live:
    from log import log_error

    ABCLI_OBJECT_ROOT = ""


class ABCLI_QGIS_Layer(object):
    def add_raster(
        self,
        filename: str,
        layer_name: str = "",
        template_name: str = "",
        set_nodata: bool = False,
        nodata: int = 0,
        verbose: bool = True,
    ) -> bool:
        layer_name = layer.get_layer_name(filename, layer_name)

        if self.exists(layer_name, verbose):
            return True

        layer_ = QgsRasterLayer(filename, layer_name)
        if not layer_.isValid():
            log_error(f"invalid filename: {filename}")
            return False

        QgsProject.instance().addMapLayer(layer_)

        if layer_ and layer_.dataProvider() and set_nodata:
            layer_.dataProvider().setNoDataValue(1, nodata)
            layer_.dataProvider().reloadData()
            layer_.triggerRepaint()

        if template_name:
            template_layer = QgsProject.instance().mapLayersByName(template_name)
            if not len(template_layer):
                log_error(f"template {template_name} not found for layer {layer_name}.")
                return False

            # https://gis.stackexchange.com/a/357206/210095
            source_style = QgsMapLayerStyle()
            source_style.readFromLayer(template_layer[0])
            source_style.writeToLayer(layer_)
            layer_.triggerRepaint()

        log(
            layer_name,
            template_name if verbose else "",
            icon="🎨",
        )

        return True

    def add_vector(
        self,
        filename: str,
        layer_name: str = "",
        template_name: str = "",
        verbose: bool = True,
    ) -> bool:
        layer_name = layer.get_layer_name(filename, layer_name)

        if self.exists(layer_name, verbose):
            return True

        layer_ = QgsVectorLayer(filename, layer_name, "ogr")
        if not layer_.isValid():
            log_error(f"invalid layer: {filename}")
            return False

        QgsProject.instance().addMapLayer(layer_)

        if template_name:
            template_layer = QgsProject.instance().mapLayersByName(template_name)
            if not len(template_layer):
                log_error(f"{template_name} not found for {layer_name}.")
                return False

            # https://gis.stackexchange.com/a/357206/210095
            source_style = QgsMapLayerStyle()
            source_style.readFromLayer(template_layer[0])
            source_style.writeToLayer(layer_)
            layer_.triggerRepaint()

        log(
            layer_name,
            template_name if verbose else "",
            icon="🎨",
        )

    def exists(
        self,
        layer_name: str,
        verbose: bool = True,
    ) -> bool:
        layer_ = QgsProject.instance().mapLayersByName(layer_name)

        if len(layer_) > 0:
            if verbose:
                log(layer_name, icon="✅")
            return True

        return False

    def help(self):
        pass

    @property
    def filename(self):
        try:
            return iface.activeLayer().dataProvider().dataSourceUri()
        except:
            log_error("unknown layer.filename.")
            return ""

    @property
    def name(self):
        filename = self.filename
        return filename.split(os.sep)[-1].split(".")[0] if filename else ""

    @property
    def object_name(self):
        filename = self.filename
        if not filename:
            return ""

        if ABCLI_OBJECT_ROOT not in filename:
            return ""

        tokens = filename.split(f"{ABCLI_OBJECT_ROOT}/", 1)[1].split("/")
        if not tokens:
            return ""

        return tokens[0]

    @property
    def path(self):
        return os.path.dirname(self.filename)


layer = ABCLI_QGIS_Layer()



from typing import List

from blue_options.terminal import show_usage, xtra
from blue_options import env

from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo.datacube.modalities import options as modality_options


def ingest_options(mono: bool) -> str:
    return "".join(
        [
            xtra("~copy_template,dryrun,overwrite,", mono),
            "scope=<scope>",
            xtra(",upload", mono),
        ]
    )


def help_crop(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "download,dryrun,suffix=<suffix>"
    return show_usage(
        [
            "@datacube crop",
            f"[{options}]",
            "[..|<object-name>]",
            "[.|<datacube-id>]",
        ],
        "crop <datacube-id> by <object-name>/target/shape.geojson -> <datacube-id>-DERIVED-crop-<suffix>.",
        mono=mono,
    )


def help_generate(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun", mono)

    args = [
        "[--modality <{}>]".format("|".join(modality_options)),
        "[--overwrite <1>]",
    ]

    return show_usage(
        [
            "@datacube generate",
            f"[{options}]",
            "[.|<datacube-id>]",
        ]
        + args,
        "generate datacube/<modality>.",
        mono=mono,
    )


def help_get(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "catalog|raw|template"

    return show_usage(
        [
            "@datacube get",
            f"[{options}]",
            "[.|<datacube-id>]",
        ],
        "get datacube properties.",
        mono=mono,
    )


def help_ingest(
    tokens: List[str],
    mono: bool,
) -> str:
    options = ingest_options(mono)

    return show_usage(
        [
            "@datacube ingest",
            f"[{options}]",
            "[.|<datacube-id>]",
            "[<args>]",
        ],
        "ingest <datacube-id>/<scope>.",
        {
            f"scope={DatacubeScope.help}": [
                "all: ALL files.",
                "metadata (default): any < 1 MB.",
                "raster: all raster.",
                "rgb: rgb.",
                "rgbx: rgb and what is needed to build rgb.",
                "<suffix>: any *<suffix>.",
            ]
        },
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--count 1]",
        "[--delim +]",
        "[--exists 1]",
        "[--log 0]",
        f"[--scope {DatacubeScope.help}]",
    ]

    return show_usage(
        [
            "@datacube list",
            "[.|<datacube-id>]",
        ]
        + args,
        "list datacube files.",
        mono=mono,
    )


help_functions = {
    "crop": help_crop,
    "generate": help_generate,
    "get": help_get,
    "ingest": help_ingest,
    "list": help_list,
}



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_geo import NAME
from blue_geo.catalog.default import add_default_arguments
from blue_geo.catalog import get_datacube_class_in_catalog
from blue_geo.catalog.generic.generic.stac import STACDatacube
from blue_geo.catalog.copernicus.sentinel_2.classes import CopernicusSentinel2Datacube
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="query",
)
add_default_arguments(STACDatacube.query_args, parser)
parser.add_argument(
    "--object_name",
    type=str,
    default="",
)
parser.add_argument(
    "--catalog",
    type=str,
)
parser.add_argument(
    "--collection",
    type=str,
)
args = parser.parse_args()


success = False
if args.task == "query":
    datacube_class = get_datacube_class_in_catalog(
        args.catalog,
        args.collection,
    )

    assert issubclass(datacube_class, STACDatacube)

    success = datacube_class.query(
        object_name=args.object_name,
        bbox=(
            [float(item) for item in args.bbox.split(",") if item]
            if args.bbox
            else [
                args.lon - args.radius,
                args.lat - args.radius,
                args.lon + args.radius,
                args.lat + args.radius,
            ]
        ),
        datetime=args.datetime,
        count=args.count,
        keyword=args.keyword,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import os
import xml.etree.ElementTree as ET
import shutil
from typing import List


def list_of_dependencies(
    filename: str,
    ABCLI_OBJECT_ROOT: str,
    verbose: bool = False,
) -> List[str]:
    file_path = os.path.dirname(filename)

    if filename.endswith(".qgz"):
        temp_dir = os.path.join(file_path, "_QGIS_temp")
        os.makedirs(temp_dir, exist_ok=True)

        os.system(f'unzip -q -o "{filename}" -d "{temp_dir}"')

        found = False
        for filename_ in os.listdir(temp_dir):
            if filename_.endswith(".qgs"):
                filename = filename.replace(".qgz", ".qgs")
                shutil.copy(os.path.join(temp_dir, filename_), filename)
                found = True
        shutil.rmtree(temp_dir)
        if not found:
            return []

    # Load QGS file as XML
    tree = ET.parse(filename)
    root = tree.getroot()

    layer_elements = root.findall(".//maplayer")

    list_of_objects = []
    for layer_element in layer_elements:
        datasource = layer_element.find("datasource")
        if datasource is None or not datasource.text:
            continue

        layer_filename = os.path.abspath(
            os.path.join(file_path, os.path.dirname(datasource.text))
        )
        if verbose:
            print(f"skipped {layer_filename}")

        if ABCLI_OBJECT_ROOT not in layer_filename:
            if verbose:
                print(f"skipped {layer_filename}")
            continue

        tokens = layer_filename.split(f"{ABCLI_OBJECT_ROOT}/", 1)[1].split("/")
        if not tokens:
            if verbose:
                print(f"skipped {layer_filename}")
            continue

        object_name = tokens[0]
        if verbose:
            print(f"included {object_name}: {layer_filename}")
        list_of_objects += [object_name]

    return sorted(list(set(list_of_objects)))
